package sqlsession;

import executor.SimpleExecutor;
import lombok.AllArgsConstructor;
import model.Configuration;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Proxy;
import java.lang.reflect.Type;
import java.util.List;

@AllArgsConstructor
public class DefaultSqlSession implements SqlSession {
  private Configuration configuration;

  public <T> T select(String statementId, boolean isList, Object... params) throws Exception {
    SimpleExecutor simpleExecutor = new SimpleExecutor();
    List<Object> resultList = simpleExecutor.select(configuration, statementId, params);
    if (!isList) {
      if (resultList.size() > 1) {
        throw new RuntimeException("查询结果过多");
      } else if (resultList.size() < 1) {
        // 无查询结果，返回null
        return null;
      } else {
        return (T) resultList.get(0);
      }
    }
    return (T) resultList;
  }

  /** 通过jdk动态代理生成mapper的代理对象，则无需自定义mapper实现类 */
  public <T> T getMapper(Class<?> mapperClass) {
    Object proxyInstance =
        Proxy.newProxyInstance(
            DefaultSqlSession.class.getClassLoader(),
            new Class[] {mapperClass},
            new InvocationHandler() {
              public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
                // 获取方法名称
                String methodName = method.getName();
                // 获取方法所在类名称
                String className = method.getDeclaringClass().getName();
                // 获取statementId
                String statementId = className + "." + methodName;

                // 获取返回值类型
                Type genericReturnType = method.getGenericReturnType();
                // 根据类型是否是泛型判断返回结果是否list
                boolean isList = genericReturnType instanceof ParameterizedType;
                return select(statementId, isList, args);
              }
            });
    return (T) proxyInstance;
  }
}
